/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <MapUnidraw/mapabout.h>
#include <MapUnidraw/mapclasses.h>
#include <OverlayUnidraw/oved.h>
#include <InterViews/action.h>
#include <InterViews/dialog.h>
#include <InterViews/layout.h>
#include <InterViews/polyglyph.h>
#include <InterViews/session.h>
#include <InterViews/window.h>
#include <IV-look/kit.h>
#include <string.h>

class MapAboutDialog : public Dialog {
public:
  MapAboutDialog(Glyph*, Style*);
  void dismiss();
};

MapAboutDialog::MapAboutDialog(Glyph* g, Style* s) 
: Dialog(g,s)
{
}

void MapAboutDialog::dismiss() {
    Dialog::dismiss(true);
}

declareActionCallback(MapAboutDialog)
implementActionCallback(MapAboutDialog)

ClassId MapAboutCmd::GetClassId () { return MAPABOUT_CMD; }

boolean MapAboutCmd::IsA (ClassId id) {
    return MAPABOUT_CMD == id || Command::IsA(id);
}

MapAboutCmd::MapAboutCmd (ControlInfo* c) : Command(c) { Init(); }
MapAboutCmd::MapAboutCmd (Editor* ed) : Command(ed) { Init(); }

void MapAboutCmd::Init() {
    const LayoutKit& layout = *LayoutKit::instance();
    const WidgetKit& kit = *WidgetKit::instance();
    PolyGlyph* vb = layout.vbox(25);
    char banner[] = "\
/*\n\
 * binary Copyright 1998 Vectaport Inc.\n\
 *\n\
 * Permission to use, copy, modify, distribute, and sell this software and\n\
 * its documentation for any purpose is hereby granted without fee, provided\n\
 * that the above copyright notice appear in all copies and that both that\n\
 * copyright notice and this permission notice appear in supporting\n\
 * documentation, and that the names of the copyright holders not be used in\n\
 * advertising or publicity pertaining to distribution of the software\n\
 * without specific, written prior permission.  The copyright holders make\n\
 * no representations about the suitability of this software for any purpose.\n\
 * It is provided \"as is\" without express or implied warranty.\n\
 *\n\
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS\n\
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.\n\
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,\n\
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING\n\
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,\n\
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION\n\
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.\n\
 *\n\
 */\n\
 \n\
source and online documentation at http://www.vectaport.com/ivmaps/\n\
 ";
    vb->append(kit.label(strtok(banner, "\n")));
    char* line = strtok(nil, "\n");
    while (line) {
	vb->append(kit.label(line));
	line = strtok(nil, "\n");
    }
    vb->append(layout.vglue(5));
    Button* but;
    aboutdialog = new MapAboutDialog(kit.outset_frame(layout.margin(vb,5.0)), Session::instance()->style());
    Action* act = new ActionCallback(MapAboutDialog)(this->aboutdialog, &MapAboutDialog::dismiss);
    vb->append(layout.hbox(layout.hglue(), but = kit.push_button("OK", act), layout.hglue()));
    aboutdialog->append_input_handler(but);
    aboutdialog->focus(but);
    Resource::ref(aboutdialog);
}

Command* MapAboutCmd::Copy () {
    Command* copy = new MapAboutCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapAboutCmd::Execute () {
    OverlayEditor* editor = (OverlayEditor*)GetEditor();
    aboutdialog->post_for(editor->GetWindow());
}

boolean MapAboutCmd::Reversible () { return true; }
