/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapfeature.h>

#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mapviews.h>
#include <MapUnidraw/mapps.h>

#include <OverlayUnidraw/ovarrow.h>
#include <OverlayUnidraw/ovellipse.h>
#include <OverlayUnidraw/ovline.h>
#include <OverlayUnidraw/ovpolygon.h>
#include <OverlayUnidraw/ovtext.h>

#include <Unidraw/Commands/command.h>

#include <Unidraw/editor.h>

#include <iostream.h>


/****************************************************************************/

ClassId MapFeaturePS::GetClassId () { return MAPFEATURE_PS; }

boolean MapFeaturePS::IsA (ClassId id) {
    return MAPFEATURE_PS == id || OverlayPS::IsA(id);
}

MapFeaturePS::MapFeaturePS (MapFeatureComp* subj) : OverlayPS(subj) {
}

boolean MapFeaturePS::Definition (ostream& out) {
    MapFeatureComp* comp = (MapFeatureComp*)GetGraphicComp();
    MapFeature* feature = comp->feature();
    Viewer* viewer = GetCommand()->GetEditor()->GetViewer();
    MapFeatureView* view = (MapFeatureView*)comp->FindView(viewer);
    Command *cmd = GetCommand();
    if (cmd != nil && cmd->IsA(MAPEXPORT_CMD))
	if ( !((MapExportCmd*)cmd)->Selected(view) )
	    return true;
    OverlayPS* ovpsv = nil;
    OverlayComp* ovcomp = new OverlayComp(view->GetGraphic());
    switch(feature->type()) {
    case MapFeature::PointType:
	ovpsv = new EllipsePS(ovcomp);
	break;
    case MapFeature::LineType:
	ovpsv = new ArrowMultiLinePS(ovcomp);
	break;
    case MapFeature::AreaType:
	ovpsv = new PolygonPS(ovcomp);
	break;
    case MapFeature::TextType:
	ovpsv = new TextPS(ovcomp);
	break;
    case MapFeature::JunctionType:
    case MapFeature::RasterType:
    case MapFeature::DemType:
    default:
	cerr << "Unsupported (as of yet) map feature\n";
	return 0;
    }
    ovpsv->Definition(out);
    delete ovpsv;
    ovcomp->SetGraphic(nil);
    delete ovcomp;
    return 1;
}    

MapFeatureComp* MapFeaturePS::GetMapFeatureComp () {
    return (MapFeatureComp*) GetSubject();
}
