/*
 * Copyright (c) 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef utmcmds_h
#define utmcmds_h

#include <Unidraw/Commands/command.h>
#include <Unidraw/Tools/grcomptool.h>
#include <MapUnidraw/mapcmds.h>

class OvImportCmd;

class UtmShowCmd : public Command {
public:
    UtmShowCmd(ControlInfo*);
    UtmShowCmd(Editor* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

#ifdef HAVE_ACE

// Base class for commands that connect to a server, send a command string
// and import the result

class UtmImportByServCmd : public Command {
public:
    UtmImportByServCmd(ControlInfo*);
    UtmImportByServCmd(Editor* = nil);

    void ServHost(const char* host);
    void ServPort(unsigned short port);
    void ServCom(char* com);
    void ResultImportCmd(OvImportCmd* impcmd);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
    unsigned short _port;
    char* _host;
    char* _com;
    OvImportCmd* _impcmd;
};

// Sample derived class that imports a map fclass from mapserv

class UtmMapImportByServCmd : public UtmImportByServCmd {
public:
    UtmMapImportByServCmd(ControlInfo*);
    UtmMapImportByServCmd(Editor* = nil);
    virtual void Execute();
    virtual Command* Copy();
};

#endif

class UtmPosTool : public GraphicCompTool {
public:
    UtmPosTool(ControlInfo* = nil);

    virtual Command* InterpretManipulator(Manipulator*);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
};

#define LOS_2VALUED 0
#define LOS_4VALUED 1
#define LOS_VIEWSHED 0
#define LOS_PROFILE  1


#ifdef HAVE_ACE

class UtmLosImportByServCmd : public UtmImportByServCmd {
public:
    UtmLosImportByServCmd(ControlInfo*);
    UtmLosImportByServCmd(Editor* = nil);
    virtual void Execute();
    virtual Command* Copy();

    void LOSFunc(char*);
    void DemPath(char*);
    void StartAzimuth(float);
    void EndAzimuth(float);
    void ObserverX(float);
    void ObserverY(float);
    void ObserverHeight(float);
    void TargetX(float);
    void TargetY(float);
    void TargetHeight(float);
    void TargetWidth(float);
    void Range(float);
    void ValueType(int);
    void LOSType(int);
protected:
  char* _losfunc;
  char* _dempath;
  float _startaz, _endaz, _obsx, _obsy, _obshgt, _targx, _targy,
    _targhgt, _targwid, _range;
  int _valtype, _lostype;
};


class UtmLosTool : public GraphicCompTool {
public:
    UtmLosTool(ControlInfo* = nil);

    virtual Command* InterpretManipulator(Manipulator*);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
};

class UtmPlayRouteCmd : public Command {
public:
  UtmPlayRouteCmd(ControlInfo*);
  virtual void Execute();
    virtual Command* Copy();
protected:
};

#endif /* defined(HAVE_ACE) */

class UtmOpenCmd : public MapOpenCmd {
public:
    UtmOpenCmd(ControlInfo*, OpenFileChooser* = nil);
    UtmOpenCmd(Editor* = nil, OpenFileChooser* = nil);

    virtual void Execute();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

#ifdef HAVE_ACE
class UtmElevImportCmd : public UtmImportByServCmd {
public:
  UtmElevImportCmd(ControlInfo*);
  UtmElevImportCmd(Editor* =nil);
    virtual void Execute();
    virtual Command* Copy();
    void ElevFunc(char*);
    void DemPath(char*);
    void ObserverX(float);
    void ObserverY(float);
protected:
  char* _elevfunc;
  char* _dempath;
  float _obsx, _obsy;
};

class UtmElevTool : public GraphicCompTool {
public:
    UtmElevTool(ControlInfo* = nil);

    virtual Command* InterpretManipulator(Manipulator*);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
};

class UtmElevGridImportCmd : public UtmImportByServCmd {
public:
  UtmElevGridImportCmd(ControlInfo*);
  UtmElevGridImportCmd(Editor* =nil);
    virtual void Execute();
    virtual Command* Copy();
    void ElevFunc(char*);
    void DemPath(char*);
    void Corners(float x0, float y0, float x1, float y1);
protected:
  char* _elevfunc;
  char* _dempath;
  float _x0, _y0, _x1, _y1;
};

class UtmElevGridTool : public GraphicCompTool {
public:
    UtmElevGridTool(ControlInfo* = nil);

    virtual Command* InterpretManipulator(Manipulator*);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
};
#endif

#endif
