/*
 * Copyright 1997 Vectaport Inc.
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 * Copyright 1993 David B. Hollenbeck
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * UtmKit definitions
 */

#include <UtmUnidraw/utmcmds.h>
#include <UtmUnidraw/utmkit.h>
#include <UtmUnidraw/utmeditor.h>

#include <MapUnidraw/gscmd.h>
#include <MapUnidraw/mapabout.h>
#include <MapUnidraw/mapcamera.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mapimport.h>
#include <MapUnidraw/mapinspect.h>
#include <MapUnidraw/mappoint.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/maproute.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>
#include <MapUnidraw/maptools.h>

#include <OverlayUnidraw/ovcmds.h>
#include <OverlayUnidraw/ovcomps.h>
#include <OverlayUnidraw/ovdoer.h>
#include <OverlayUnidraw/oved.h>
#include <OverlayUnidraw/ovimport.h>
#include <OverlayUnidraw/ovprint.h>
#include <OverlayUnidraw/ovviewer.h>

#include <IVGlyph/importchooser.h>

#define iv2_6_compatible 1
#include <UniIdraw/idcmds.h>
#include <UniIdraw/idkybd.h>
#undef iv2_6_compatible

#include <Unidraw/unidraw.h>
#include <Unidraw/editor.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/uctrls.h>
#include <Unidraw/keymap.h>
#include <Unidraw/kybd.h>
#include <Unidraw/catalog.h>
#include <Unidraw/viewer.h>

#include <Unidraw/Commands/catcmds.h>

#include <Unidraw/Graphic/pspaint.h>

#include <Unidraw/Tools/magnify.h>
#include <Unidraw/Tools/select.h>

#include <IV-X11/xcolor.h>
#include <IV-X11/xdisplay.h>
#include <IV-X11/xpattern.h>
#include <IV-look/kit.h>
#include <InterViews/action.h>
#include <InterViews/background.h>
#include <InterViews/display.h>
#include <InterViews/label.h>
#include <InterViews/layout.h>
#include <InterViews/patch.h>
#include <InterViews/style.h>
#include <InterViews/session.h>

#include <IVGlyph/exportchooser.h>
#include <IVGlyph/figure.h>
#include <IVGlyph/toolbutton.h>

#include <Map/mapdb.h>
#include <Map/mapelt.h>
#include <Map/maplib.h>
#include <MapUnidraw/mapeditor.h>

#include <OS/math.h>

#include <stdlib.h>
#include <string.h>

implementTable(UtmFClassMenuTable,MapFeatureClass*,MenuItem*)
implementTable(UtmFClassTable,MapFeatureClass*,boolean)

/******************************************************************************/

UtmKit* UtmKit::_utmkit = nil;

UtmKit::UtmKit()
: MapKit()
{
    InitFClassState();
}

UtmKit::~UtmKit() {
    delete _fcmenutable;
    delete _fctable;
}

UtmKit* UtmKit::Instance() {
    if (!_utmkit)
	_utmkit = new UtmKit();
    return _utmkit;
}

Glyph* UtmKit::MakeToolbar() {
    WidgetKit& kit = *WidgetKit::instance();
    const LayoutKit& layout = *LayoutKit::instance();
    Style* s = kit.style();

    ToolButton* magnify;
    ToolButton* mapselect;
    ToolButton* inspect;
#ifdef HAVE_ACE
    ToolButton* camerapos;
    ToolButton* camerapath;
#endif
    ToolButton* utmposbutton;
#ifdef HAVE_ACE
    ToolButton* elevbutton;
    ToolButton* elgrbutton;
    ToolButton* losbutton;
#endif
    ToolButton* drtbutton;
    ToolButton* prtbutton;
    ToolButton* lftbutton;
    _toolbars = layout.deck(2);

    PolyGlyph* vb = layout.vbox(10);

    TelltaleGroup* tg = new TelltaleGroup();

    Glyph* mag = kit.label("Magnify");
    Glyph* mps = kit.label("Select");
    Glyph* ins = kit.label("Inspect");
    Glyph* campos = kit.label("CameraPos");
    Glyph* campth = kit.label("CameraPath");
    Glyph* utmpos = kit.label("UTMPos");
    Glyph* los = kit.label("LOS");
    Glyph* elv = kit.label("Elev");
    Glyph* elg = kit.label("ElevGrid");
    Glyph* drt = kit.label("DrawRoute");
    Glyph* prt = kit.label("PickRoute");
    Glyph* lft = kit.label("LineFeature");

    Coord maxwidth = 0;
    Requisition req;
    maxwidth = Math::max((mag->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((mps->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((ins->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((campos->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((campth->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((utmpos->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((los->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((elv->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((elg->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((drt->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((prt->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((lft->request(req), req.x_requirement().natural()),
			 maxwidth);

    vb->append(magnify = MakeTool(new MagnifyTool(new ControlInfo("Magnify", KLBL_MAGNIFY,CODE_MAGNIFY)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(mag)), tg));
    vb->append(mapselect = MakeTool(new SelectTool(new ControlInfo("Select", "s", "s")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(mps)), tg));
    vb->append(inspect = MakeTool(new MapInspectTool(((MapEditor*)_ed)->selstate(),
					   new ControlInfo("Inspect", "i", "i")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(ins)), tg));
#ifdef HAVE_ACE
    vb->append(camerapos = MakeTool(new CameraPosTool(
	new ControlInfo("CameraPos", "c", "c")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(campos)), tg));
    vb->append(camerapath = MakeTool(new CameraPathTool(
	new ControlInfo("CameraPath", "C", "C")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(campth)), tg));
#endif
    vb->append(utmposbutton = MakeTool(new UtmPosTool(
	new ControlInfo("UTMPos")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(utmpos)), tg));
#ifdef HAVE_ACE
    vb->append(losbutton = MakeTool(new UtmLosTool(
	new ControlInfo("LOS")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(los)), tg));
    vb->append(elevbutton = MakeTool(new UtmElevTool(
	new ControlInfo("Elev")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(elv)), tg));
    vb->append(elgrbutton = MakeTool(new UtmElevGridTool(
	new ControlInfo("ElevGrid")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(elg)), tg));
#endif
    vb->append(drtbutton = MakeTool(new RouteTool(new ControlInfo("DrawRoute")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(drt)), tg));
    vb->append(prtbutton = MakeTool(new PickRouteTool(new ControlInfo("PickRoute")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(prt)), tg));
    vb->append(prtbutton = MakeTool(new LineFeatureTool(new ControlInfo("LineFeature")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(lft)), tg));

    _toolbars->append(vb);

    vb = layout.vbox(10);
    vb->append(magnify);
    vb->append(mapselect);
    vb->append(inspect);
#ifdef HAVE_ACE
    vb->append(camerapos);
    vb->append(camerapath);
#endif
    vb->append(utmposbutton);
#ifdef HAVE_ACE
    vb->append(losbutton);
    vb->append(elevbutton);
    vb->append(elgrbutton);
#endif
    vb->append(drtbutton);
    vb->append(prtbutton);
    _toolbars->append(vb);
    _toolbars->flip_to(1);
    _toolbar = new Patch(_toolbars);

    return layout.hbox(
	layout.vflexible(
	    layout.vnatural(
		new Background(
		    layout.vcenter(
			_toolbar
		    ),
		    unidraw->GetCatalog()->FindColor("#aaaaaa")
		),
		550
	    )
	)
    );
}

Glyph* UtmKit::MakeMenus()
{
    LayoutKit& lk = *LayoutKit::instance();
    Menu* menubar_ = WidgetKit::instance()->menubar();
    
    menubar_->append_item(MakeFileMenu());
    menubar_->append_item(MakeEditMenu());
    menubar_->append_item(MakeStructureMenu());
    MenuItem* m = MakeFrameMenu();
    if (m) menubar_->append_item(m);
    menubar_->append_item(MakeViewMenu());
#ifdef HAVE_ACE
    menubar_->append_item(MakeRouteMenu());
#endif
    Resource::ref(menubar_);
    return LayoutKit::instance()->vbox(lk.hnatural(menubar_, 600));
}

MenuItem * UtmKit::MakeFileMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("File"));
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, new MapAboutCmd(new ControlInfo("About ivmaps utmviewer", KLBL_ABOUT, CODE_ABOUT)),
	     "About ivmaps utmviewer");
    MakeMenu(mbi, new OvNewCompCmd(new ControlInfo("New", KLBL_NEWCOMP, CODE_NEWCOMP),
				 new MapIdrawComp),
	     "New   ");
    MakeMenu(mbi, new OvRevertCmd(new ControlInfo("Revert", KLBL_REVERT, CODE_REVERT)),
	     "Revert   ");
#if 0
    MakeMenu(mbi, new OvOpenCmd(new ControlInfo("Open...", KLBL_VIEWCOMP, CODE_VIEWCOMP)),
	     "Open...   ");
#else
    MakeMenu(mbi, new UtmOpenCmd(new ControlInfo("Open...", KLBL_VIEWCOMP, CODE_VIEWCOMP)),
	     "Open...   ");
#endif
    MakeMenu(mbi, new OvSaveCompCmd(new ControlInfo("Save", KLBL_SAVECOMP, CODE_SAVECOMP)),
	     "Save   ");
    MakeMenu(mbi, new OvSaveCompAsCmd(new ControlInfo("Save As...",
					    KLBL_SAVECOMPAS,
					    CODE_SAVECOMPAS)),
	     "Save As...   ");
    MakeMenu(mbi, new OvPrintCmd(new ControlInfo("Print...", KLBL_PRINT, CODE_PRINT)),
	     "Print...   ");
    Style * istyle = new Style(Session::instance()->style());
    istyle->attribute("subcaption", "Import map features from file:");
    istyle->attribute("open", "Import");
    ImportChooser* ichooser = new ImportChooser(".", WidgetKit::instance(), istyle, nil, false, true);
    Resource::ref(ichooser);
    MakeMenu(mbi, new MapImportCmd(new ControlInfo("Import Map Features...",
						  KLBL_IMPORT,
						  CODE_IMPORT),
				   ichooser),
	     "Import Map Features...   ");
    Style * estyle = new Style(Session::instance()->style());
    estyle->attribute("subcaption", "Export selected map features to file:");
    estyle->attribute("open", "Export");
    const char *formats[] = {"idraw", "drawtool"};
    ExportChooser* echooser = new ExportChooser(".", WidgetKit::instance(), estyle,
				 formats, sizeof(formats)/sizeof(char*), nil, nil, true, false);
    Resource::ref(echooser);
    MakeMenu(mbi, new MapExportCmd(new ControlInfo("Export Map Features...",
						  "^X", "\030"),
			       echooser),
	     "Export Map Features...   ");
    MakeMenu(mbi, new OvWindowDumpAsCmd(new ControlInfo("Dump Window As..."
						  )),
	     "Dump Window As...   ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new OvQuitCmd(new ControlInfo("Quit", KLBL_QUIT, CODE_QUIT)),
	     "Quit   ");

    return mbi;
}

MenuItem* UtmKit::MakeRouteMenu() {
#ifdef HAVE_ACE
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Route"));
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, 
	     new UtmPlayRouteCmd(new ControlInfo("Play Route")),
	     "Play Route");

    return mbi;
#endif
}

MenuItem* UtmKit::MakeImportMenu() {
#ifdef HAVE_ACE
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Import"));
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, 
	     new UtmMapImportByServCmd(new ControlInfo("Import from mapserv")),
	     "Import from mapserv");

    return mbi;
#endif
}

MenuItem* UtmKit::MakeMapMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Map"));
    mbi->menu(kit.pulldown());

    _layermenu = MakeLayerMenus();
    if (_layermenu)
        mbi->menu()->append_item(_layermenu);

    _mapmenu = mbi;
    return mbi;
}

void UtmKit::MakeCheckMenu(MenuItem* mbi, Command* scmd, Command* hcmd,
			   const char* label, MapFeatureClass* fc) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* item;
    boolean oldval;
    _ed->GetKeyMap()->Register(new CommandControl(scmd->GetControlInfo()));
    scmd->SetEditor(_ed);
    _ed->GetKeyMap()->Register(new CommandControl(hcmd->GetControlInfo()));
    hcmd->SetEditor(_ed);
    MenuItem* menu_item = kit.check_menu_item(kit.label(label));
    _fcmenutable->insert(fc, menu_item);
    ShowHideDoer* doer = new ShowHideDoer(scmd, hcmd, menu_item->state());
    menu_item->action(new ActionCallback(ShowHideDoer)(doer, &ShowHideDoer::Do));
    if (_fctable->find(oldval, fc) && oldval)
	menu_item->state()->set(TelltaleState::is_chosen, true);
    mbi->menu()->append_item(menu_item);
}
 
MenuItem* UtmKit::MakeLayerMenus() {
#if 0
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* showi = kit.menu_item(kit.label(   "Layers                     "));
    Menu* show = kit.pullright();
    showi->menu(show);
    UtmEditor* ed = (UtmEditor*)GetEditor();
    MapDatabase* db = ed->GetMapDatabase();
    UtmLibrary* lib = db->library(ed->libstate()->maplib());
    if (lib) {
	int maxcovlen =0;
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    UtmCoverage* cov = lib->coverage(ic);
	    int len = strlen(cov->description());
	    if (len > maxcovlen)
		maxcovlen = len;
	}
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    UtmCoverage* cov = lib->coverage(ic);
	    char covdesc[maxcovlen+1];
	    memset(covdesc, ' ', maxcovlen);
	    covdesc[maxcovlen] = '\0';
	    strncpy(covdesc, cov->description(), strlen(cov->description()));
	    MenuItem* showcovi = kit.menu_item(kit.label(covdesc));
	    Menu* showcovm = kit.pullright();
	    showcovi->menu(showcovm);
	    int maxfclen =0;
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		MapFeatureClass* fc = cov->feature_class(ifc);
		int len = strlen(fc->description());
		if (len > maxfclen)
		    maxfclen = len;
	    }
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		MapFeatureClass* fc = cov->feature_class(ifc);
		MapShowCmd* showcmd = new MapShowCmd(new ControlInfo(fc->description()), false);
		showcmd->SetLibName(lib->name());
		showcmd->SetCovName(cov->name());
		showcmd->SetFClassName(fc->name());
		MapHideCmd* hidecmd = new MapHideCmd(new ControlInfo(fc->description()), false);
		hidecmd->SetLibName(lib->name());
		hidecmd->SetCovName(cov->name());
		hidecmd->SetFClassName(fc->name());
		char fcdesc[maxfclen+1];
		memset(fcdesc, ' ', maxfclen);
		fcdesc[maxfclen] = '\0';
		strncpy(fcdesc, fc->description(), strlen(fc->description()));
		// if fc shown set check for menu
		MakeCheckMenu(showcovi, showcmd, hidecmd, fcdesc, fc);
	    }
	    show->append_item(showcovi);
	}
    }
    return showi;
#else
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* showi = kit.menu_item(kit.label(   "Layers                     "));
    Menu* show = kit.pullright();
    showi->menu(show);

    return showi;
#endif
}

void UtmKit::FClassState(MapFeatureClass* fc, boolean val) {
    MenuItem* item;
    boolean oldval;
    if (_fcmenutable->find(item, fc))
	item->state()->set(TelltaleState::is_chosen, val);
    _fctable->insert(fc, val);
}

void UtmKit::InitFClassState() {
    if (_fcmenutable)
        delete _fcmenutable;
    _fcmenutable = new UtmFClassMenuTable(100);
    if (_fctable)
        delete _fctable;
    _fctable = new UtmFClassTable(100);    
}

MenuItem* UtmKit::MakeLibraryMenu() {
#if 0
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* libi = kit.menu_item(kit.label(   "Library                   "));
    Menu* libm = kit.pullright();
    libi->menu(libm);
    UtmEditor* ed = (UtmEditor*)GetEditor();
    UtmDatabase* db = ed->GetMapDatabase();
    int maxliblen = 0;
    for (int il = 0; il < db->nlibs(); il++) {
	UtmLibrary* lib = db->library(il);
	int len = strlen(lib->name());
	if (len > maxliblen)
	    maxliblen = len;
    }
    TelltaleGroup* tg = new TelltaleGroup();
    for (int il = 0; il < db->nlibs(); il++) {
	UtmLibrary* lib = db->library(il);
	char libname[maxliblen+1];
	memset(libname, ' ', maxliblen);
	libname[maxliblen] = '\0';
	strncpy(libname, lib->name(), strlen(lib->name()));
	MenuItem* menu_item = kit.radio_menu_item(tg, libname);
	if (strcmp(lib->name(), ed->libstate()->maplib())==0)
	    menu_item->state()->set(TelltaleState::is_chosen, true);
	MapLibCmd* cmd = new MapLibCmd(new ControlInfo(""), lib->name());
	cmd->SetEditor(ed);
	CommandPusher* pusher = new CommandPusher(cmd);
	menu_item->action(new ActionCallback(CommandPusher)
			  (pusher, &CommandPusher::Push));
	libm->append_item(menu_item);
    }
    return libi;
#else
    return nil;
#endif
}

void UtmKit::UpdateLayerMenu() {
    _mapmenu->menu()->replace_item(1, MakeLayerMenus());
}

void UtmKit::UpdateLibraryMenu() {
    _mapmenu->menu()->replace_item(2, MakeLibraryMenu());
}

Glyph* UtmKit::MakeStates() {
    return nil;
}

